@@ -4,13 +4,15 @@ import re
 import stat
 import sys
 import urllib.request as urllib_request
+from contextlib import nullcontext
 
 import numpy as np
 import pytest
 
 import asdf
-from asdf import exceptions, generic_io, util
+from asdf import exceptions, generic_io
 from asdf.config import config_context
+from asdf.exceptions import AsdfDeprecationWarning
 
 from . import _helpers as helpers
 from . import create_large_tree, create_small_tree
@@ -21,6 +23,26 @@ def tree(request):
     return request.param()
 
 
+@pytest.fixture(params=[True, False])
+def has_fsspec(request, monkeypatch):
+    if request.param:
+        yield True
+    else:
+        pytest.importorskip("fsspec")
+        monkeypatch.setitem(sys.modules, "fsspec", None)
+        yield False
+
+
+@pytest.fixture()
+def warn_no_fsspec(has_fsspec):
+    if has_fsspec:
+        yield nullcontext()
+    else:
+        yield pytest.warns(
+            AsdfDeprecationWarning, match=r"Opening http urls without fsspec is deprecated. Please install fsspec"
+        )
+
+
 def _roundtrip(tree, get_write_fd, get_read_fd, write_options=None, read_options=None):
     write_options = {} if write_options is None else write_options
     read_options = {} if read_options is None else read_options
@@ -91,38 +113,38 @@ def test_open(tmp_path, small_tree):
 
 
 def test_path(tree, tmp_path):
-    path = os.path.join(str(tmp_path), "test.asdf")
+    path = tmp_path / "test.asdf"
 
     def get_write_fd():
         f = generic_io.get_file(path, mode="w")
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         return f
 
     def get_read_fd():
         # Must open with mode=rw in order to get memmapped data
         f = generic_io.get_file(path, mode="rw")
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         # This is to check for a "feature" in Python 3.x that reading zero
         # bytes from a socket causes it to stop.  We have code in generic_io.py
         # to workaround it.
         f.read(0)
         return f
 
-    with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
+    with _roundtrip(tree, get_write_fd, get_read_fd, read_options={"memmap": True}) as ff:
         assert len(ff._blocks.blocks) == 2
         assert isinstance(ff._blocks.blocks[0].cached_data, np.memmap)
 
 
 def test_open2(tree, tmp_path):
-    path = os.path.join(str(tmp_path), "test.asdf")
+    path = tmp_path / "test.asdf"
 
     def get_write_fd():
         # cannot use context manager here because it closes the file
         f = generic_io.get_file(open(path, "wb"), mode="w", close=True)
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         return f
 
     def get_read_fd():
@@ -130,17 +152,17 @@ def test_open2(tree, tmp_path):
         # cannot use context manager here because it closes the file
         f = generic_io.get_file(open(path, "r+b"), mode="rw", close=True)
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         return f
 
-    with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
+    with _roundtrip(tree, get_write_fd, get_read_fd, read_options={"memmap": True}) as ff:
         assert len(ff._blocks.blocks) == 2
         assert isinstance(ff._blocks.blocks[0].cached_data, np.memmap)
 
 
 @pytest.mark.parametrize("mode", ["r", "w", "rw"])
 def test_open_not_binary_fail(tmp_path, mode):
-    path = os.path.join(str(tmp_path), "test.asdf")
+    path = tmp_path / "test.asdf"
 
     with open(path, "w") as fd:
         fd.write("\n\n\n")
@@ -157,23 +179,23 @@ def test_open_not_binary_fail(tmp_path, mode):
 
 
 def test_io_open(tree, tmp_path):
-    path = os.path.join(str(tmp_path), "test.asdf")
+    path = tmp_path / "test.asdf"
 
     def get_write_fd():
         # cannot use context manager here because it closes the file
         f = generic_io.get_file(open(path, "wb"), mode="w", close=True)
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         return f
 
     def get_read_fd():
         # cannot use context manager here because it closes the file
         f = generic_io.get_file(open(path, "r+b"), mode="rw", close=True)
         assert isinstance(f, generic_io.RealFile)
-        assert f._uri == util.filepath_to_url(path)
+        assert f._uri == path.as_uri()
         return f
 
-    with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
+    with _roundtrip(tree, get_write_fd, get_read_fd, read_options={"memmap": True}) as ff:
         assert len(ff._blocks.blocks) == 2
         assert isinstance(ff._blocks.blocks[0].cached_data, np.memmap)
         ff.tree["science_data"][0] = 42
@@ -259,7 +281,7 @@ def test_urlopen(tree, httpserver):
 
 
 @pytest.mark.remote_data()
-def test_http_connection(tree, httpserver):
+def test_http_connection(tree, httpserver, warn_no_fsspec):
     path = os.path.join(httpserver.tmpdir, "test.asdf")
 
     def get_write_fd():
@@ -274,9 +296,10 @@ def test_http_connection(tree, httpserver):
         fd.read(0)
         return fd
 
-    with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
-        assert len(ff._blocks.blocks) == 2
-        assert (ff.tree["science_data"] == tree["science_data"]).all()
+    with warn_no_fsspec:
+        with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
+            assert len(ff._blocks.blocks) == 2
+            assert (ff.tree["science_data"] == tree["science_data"]).all()
 
 
 def test_exploded_filesystem(tree, tmp_path):
@@ -313,7 +336,7 @@ def test_exploded_filesystem_fail(tree, tmp_path):
 
 
 @pytest.mark.remote_data()
-def test_exploded_http(tree, httpserver):
+def test_exploded_http(tree, httpserver, warn_no_fsspec):
     path = os.path.join(httpserver.tmpdir, "test.asdf")
 
     def get_write_fd():
@@ -322,8 +345,9 @@ def test_exploded_http(tree, httpserver):
     def get_read_fd():
         return generic_io.get_file(httpserver.url + "test.asdf")
 
-    with _roundtrip(tree, get_write_fd, get_read_fd, write_options={"all_array_storage": "external"}) as ff:
-        assert len(list(ff._blocks.blocks)) == 0
+    with warn_no_fsspec:
+        with _roundtrip(tree, get_write_fd, get_read_fd, write_options={"all_array_storage": "external"}) as ff:
+            assert len(list(ff._blocks.blocks)) == 0
 
 
 def test_exploded_stream_write(small_tree):
@@ -378,7 +402,7 @@ def test_open_stdout():
         pass
 
 
-def test_invalid_obj(tmp_path):
+def test_invalid_obj(tmp_path, has_fsspec):
     with pytest.raises(ValueError, match=r"Can't handle .* as a file for mode 'r'"):
         generic_io.get_file(42)
 
@@ -392,8 +416,14 @@ def test_invalid_obj(tmp_path):
     ):
         generic_io.get_file(fd, "r")
 
-    with pytest.raises(ValueError, match=r"HTTP connections can not be opened for writing"):
-        generic_io.get_file("http://www.google.com", "w")
+    url = "http://www.google.com"
+    mode = "w"
+    if has_fsspec:
+        raises_ctx = pytest.raises(ValueError, match=f"Unable to open {url} with mode {mode}")
+    else:
+        raises_ctx = pytest.raises(ValueError, match=r"HTTP connections can not be opened for writing")
+    with raises_ctx:
+        generic_io.get_file(url, mode)
 
     with pytest.raises(TypeError, match=r"io.StringIO objects are not supported.  Use io.BytesIO instead."):
         generic_io.get_file(io.StringIO())
